@@ -6,7 +6,7 @@ import logging
 
 import pandas as pd
 
-def make_shards_df(glob = None, df = None, header = 'infer', sep = ','): 
+def make_partitions_df(glob = None, df = None, header = 'infer', sep = ','): 
     assert glob is not None or df is not None, "Either glob or df must be specified"
     df = pd.read_csv(f"filecache::{glob}", sep = sep, header = header) if glob else df
     assert set(df.columns) == set(['id', 'count']), "The metadata source must include id and count columns"
@@ -16,7 +16,7 @@ def make_shards_df(glob = None, df = None, header = 'infer', sep = ','):
     df = df[['id', 'start_idx', 'end_idx']]
     return df
 
-def objs_to_shards_df(objs, header = None):
+def objs_to_partitions_df(objs, header = None):
     rows = []
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
@@ -83,15 +83,15 @@ class BaseObjectStorageDataset():
                  header = 'infer', 
 
                  index_shards = None,
-                 shards_glob = None, 
-                 make_shards_df_fn = make_shards_df, 
-                 objs_to_shards_df_fn = objs_to_shards_df,
+                 partitions_glob = None, 
+                 make_partitions_df_fn = make_partitions_df, 
+                 objs_to_partitions_df_fn = objs_to_partitions_df,
 
                  stats_glob = True,
 
                  worker = 0, 
                  replicas = 1, 
-                 shard_size = 1,   
+                 shard_size = None,   
                  batch_size = None,                
                  max_shards = None,
 
@@ -112,7 +112,6 @@ class BaseObjectStorageDataset():
             assert shard_size, "shard_size must be specified if replicas is greater than 1"
 
         #resolve batch_size, shard_size, and max_shards
-        assert batch_size is not None or shard_size is not None, "either batch_size or shard_size must be specified"
         if shard_size is not None and batch_size is not None:
             assert type(batch_size) is int and batch_size > 0 and type(shard_size) is int and shard_size > 0, "If batch_size and shard_size are specified, both must be greater than zero integers"
             #TODO: For now default to the smaller size
@@ -128,7 +127,12 @@ class BaseObjectStorageDataset():
             batch_size = shard_size
         elif batch_size is not None and shard_size is None:
             assert type(batch_size) is int and batch_size > 0, "The value for batch_size must be a greater than zero integer"
-            shard_size = batch_size        
+            shard_size = batch_size
+        elif batch_size is None and shard_size is None:
+            logging.warn(f"neither shard_size nor batch_size are specified, defaulting to batch_size=1 and shard_size=1")
+            batch_size = shard_size = 1
+        assert batch_size is not None or shard_size is not None, "either batch_size or shard_size must be specified"
+        
         self.shard_size = shard_size
         self.max_shards = max_shards if max_shards else float('nan')
 
@@ -180,6 +184,7 @@ class BaseObjectStorageDataset():
         #create a stats dataframe unless this feature is turned off
         if stats_glob is None or stats_glob is False:
             self.stats_df = None
+
         else:
             #if not specified explicitly try to fall back to default location
             if type(stats_glob) is not str and protocol not in ('http', 'https'):
@@ -205,7 +210,6 @@ class BaseObjectStorageDataset():
 
             else:
                 stats_glob = None
-
             
             if stats_glob is not None:
                 self.stats_df = pd.read_csv(f"filecache::{stats_glob}")
@@ -214,56 +218,56 @@ class BaseObjectStorageDataset():
         if index_shards is None:   
             
             #ignore fall back with protocols that dont support glob
-            if shards_glob is None and protocol not in ('http', 'https'):
+            if partitions_glob is None and protocol not in ('http', 'https'):
 
                 #if the glob is a file or a wildcard
                 if self.fs.isfile(glob) or (not self.fs.isfile(glob) and not self.fs.isdir(glob)):
-                    maybe_shards_glob = f"{protocol}://{self.fs._parent(glob)}/.meta/shards/"
+                    maybe_partitions_glob = f"{protocol}://{self.fs._parent(glob)}/.meta/shards/"
                 elif self.fs.isdir(glob):
                     #remove trailing / characters
-                    maybe_shards_glob = f"{remove_trailing_forward_slash(glob)}/.meta/shards/"                
+                    maybe_partitions_glob = f"{remove_trailing_forward_slash(glob)}/.meta/shards/"                
                 else:
                     logging.warn(f"failed to construct a shards dataframe path for glob={glob}")
-                    maybe_shards_glob = None
+                    maybe_partitions_glob = None
                 
-                maybe_shards = self.fs.glob(maybe_shards_glob)
+                maybe_shards = self.fs.glob(maybe_partitions_glob)
                 if not isinstance(maybe_shards, list) or not len(maybe_shards):
-                    logging.warn(f"shards_glob is not specified at initialization, failed to find shard metadata at default location {maybe_shards_glob}")
+                    logging.warn(f"partitions_glob is not specified at initialization, failed to find shard metadata at default location {maybe_partitions_glob}")
                 else:
-                    [shards_glob] = maybe_shards
-                    shards_glob = f"{protocol}://{shards_glob}"
-                    logging.warn(f"shards_glob is not specified at initialization, defaulting to shards_glob={shards_glob}")
+                    [partitions_glob] = maybe_shards
+                    partitions_glob = f"{protocol}://{partitions_glob}"
+                    logging.warn(f"partitions_glob is not specified at initialization, defaulting to partitions_glob={partitions_glob}")
 
-            if shards_glob is None:
+            if partitions_glob is None:
                 index_shards = True
-                logging.warn("shards_glob is not specified at initialization, attempting to proceed with index_shards=True which can take a while for large objects.")
+                logging.warn("partitions_glob is not specified at initialization, attempting to proceed with index_shards=True which can take a while for large objects.")
             else:
                 index_shards = False
         else:
             assert isinstance(index_shards, bool), "index_shards must be set to True or False"
 
-        assert make_shards_df_fn, "make_shards_df_fn must be specified"
-        self.make_shards_df_fn = make_shards_df_fn
+        assert make_partitions_df_fn, "make_partitions_df_fn must be specified"
+        self.make_partitions_df_fn = make_partitions_df_fn
 
         if index_shards is None \
                 or index_shards is True \
-                or shards_glob is not None:
-            assert objs_to_shards_df_fn, "objs_to_shards_df_fn must be specified"
-            self.objs_to_shards_df_fn = objs_to_shards_df_fn
+                or partitions_glob is not None:
+            assert objs_to_partitions_df_fn, "objs_to_partitions_df_fn must be specified"
+            self.objs_to_partitions_df_fn = objs_to_partitions_df_fn
 
 
         # if the shard metadata is unavailable
         # use the objects in the glob to generate the metadata
         # with the count of lines of records per shard            
         if index_shards is None or index_shards is True:            
-            self.shards_df = make_shards_df_fn(df = objs_to_shards_df_fn(self.objs, header = self.header))
+            self.partitions_df = make_partitions_df_fn(df = objs_to_partitions_df_fn(self.objs, header = self.header))
         else:            
-            self.shards_df = make_shards_df_fn(glob = shards_glob)
+            self.partitions_df = make_partitions_df_fn(glob = partitions_glob)
                 
         # if the number of shards is identical to the number of metadata records
-        if len(self.objs) == len(self.shards_df):
-            self.shards_df = self.shards_df.sort_values(by = 'id', ascending = True)
-            self.shards_df['uri'] = pd.Series(name = 'uri', data = self.objs).sort_values(ascending=True)
+        if len(self.objs) == len(self.partitions_df):
+            self.partitions_df = self.partitions_df.sort_values(by = 'id', ascending = True)
+            self.partitions_df['uri'] = pd.Series(name = 'uri', data = self.objs).sort_values(ascending=True)
 
         else:            
             # query for obj sizes in order to later ignore the empty shards 
@@ -279,23 +283,23 @@ class BaseObjectStorageDataset():
             # use just the non-empty shards / partitions
             objs_df = objs_df[ objs_df['size'] > 0 ]
 
-            assert len(objs_df) == len(self.shards_df), f"The number of the non-empty objects matching the glob pattern {len(objs_df)} is not equal to the number of the non-empty shards {len(self.shards_df)}."
+            assert len(objs_df) == len(self.partitions_df), f"The number of the non-empty objects matching the glob pattern {len(objs_df)} is not equal to the number of the non-empty shards {len(self.partitions_df)}."
 
-            self.shards_df['uri'] = objs_df['key'].apply(lambda s: f"{protocol}://{s}").sort_values(ascending=True)
+            self.partitions_df['uri'] = objs_df['key'].apply(lambda s: f"{protocol}://{s}").sort_values(ascending=True)
 
         #TODO: improve: right now assumes that the uri has the matching ID specified in 'part-00000-' substring inthe file
         # objs_df = pd.DataFrame(columns=['uri'], data = pd.Series(self.objs).apply(lambda s: f"{protocol}://{s}"))
         # id_re = re.compile('part-(\d{5})-')
         # objs_df['id'] = objs_df['uri'].apply(lambda s: int(id_re.findall(s)[0]))
-        # self.shards_df = self.shards_df.merge(objs_df, on = 'id')
+        # self.partitions_df = self.partitions_df.merge(objs_df, on = 'id')
         #TODO: improve
 
-        self.obj_count = len(self.shards_df)
-        self.row_count = self.shards_df.iloc[-1]['end_idx']
+        self.obj_count = len(self.partitions_df)
+        self.row_count = self.partitions_df.iloc[-1]['end_idx']
 
     def __len__(self):
-        if self.shards_df is not None:
-            return self.shards_df.iloc[-1]['end_idx']
+        if self.partitions_df is not None:
+            return self.partitions_df.iloc[-1]['end_idx'] + 1
         else:
             return -1
 
@@ -311,25 +315,18 @@ class BaseObjectStorageDataset():
                 row_end_indicies = list(row_start_indicies[1:] + [row_end_idx])
                 src_df_list = []
                 for row_start_idx, row_end_idx in zip(row_start_indicies, row_end_indicies):
-                    print(f"*** {row_start_idx=}, {row_end_idx=} ***")
 
                     #query rows based on actual indicies in the dataset
                     row_start_idx = row_start_idx % self.row_count
                     row_end_idx = row_end_idx % self.row_count
 
-                    print(f"*** {row_start_idx=}, {row_end_idx=} ***")
-
-                    print(f"*** {self.header=} ***")
-
                     if self.header is None:
-                        for (skiprows, nrows, uri) in df_read_spec(self.shards_df, self.row_count, row_start_idx, row_end_idx):
+                        for (skiprows, nrows, uri) in df_read_spec(self.partitions_df, self.row_count, row_start_idx, row_end_idx):
                             src_df_list.append( pd.read_csv(f"filecache::{uri}", header = None, nrows=nrows, skiprows=skiprows) )
 
                     elif self.header == 'infer' or self.header == 0:                        
-                        for (skiprows, nrows, uri) in df_read_spec(self.shards_df, self.row_count, row_start_idx, row_end_idx):
+                        for (skiprows, nrows, uri) in df_read_spec(self.partitions_df, self.row_count, row_start_idx, row_end_idx):
                             
-                            print(f"*** {skiprows=}, {nrows=}, {uri=} ***")
-
                             if (skiprows == 0):
                                 #skiprows == 0 when we are at the top of the file with a (potential) header                            
                                 df = pd.read_csv(f"filecache::{uri}", header = self.header, nrows = nrows, skiprows = 0)
@@ -355,7 +352,7 @@ class BaseObjectStorageDataset():
                         raise ValueError(f"header attribute cannot be set to {self.header}")
 
                     # map(lambda df: df.columns, src_df_list)
-                    # assert len(set(map(lambda df: df.shape[1], src_df_list))) == 1, f"Failed to process in the following fragment of the dataset {df_read_spec(self.shards_df, self.row_count, row_start_idx, row_end_idx)} due to a change in the number of the columns."
+                    # assert len(set(map(lambda df: df.shape[1], src_df_list))) == 1, f"Failed to process in the following fragment of the dataset {df_read_spec(self.partitions_df, self.row_count, row_start_idx, row_end_idx)} due to a change in the number of the columns."
 
                 yield pd.concat(
                     objs = src_df_list,
